#include "global.h"


/* Default configuration values */
struct Config cfg = {
    1,					/* INameUser	*/
    1,					/* FullURL	*/
    0,					/* ResolveIP	*/
    0,					/* FullTraffic	*/
    1,					/* ShowInfo	*/
    1,					/* Indics	*/
    0,					/* Overwrite	*/
    TMPDIR,				/* TmpDir	*/
    WWWDIR,				/* TargetDir	*/
    NULL,				/* Site		*/
    LOGFILE,				/* Log		*/
    ETCDIR"/free-sa.conf",		/* Config	*/
    NULL,				/* utFile	*/
    NULL,				/* dldFile	*/
    NULL,				/* locFile	*/
    NULL,				/* gfFile	*/
    NULL,				/* Logo		*/
    NULL,				/* Name		*/
    "",					/* Locale	*/
    1,					/* TSR		*/
    1,					/* PER		*/
    { 1, 1, 1, 1, 1, 1, 0 },		/* TTR		*/
    { 50, 50, 50, 50, 50, 50, 50 },	/* ttUserLimit	*/
    -1,					/* LogFormat	*/
    50,					/* URLLimit	*/
    5					/* AutoRecords	*/
};

struct owConfig owcfg = {
    NULL				/* Log		*/
};

struct iConfig icfg = {
    NULL,				/* ReportDir	*/
    NULL,				/* CharSet	*/
    "description",			/* DFname	*/
    "index",				/* IRname	*/
    "index",				/* LRname	*/
    "topuser",				/* TURname	*/
    "topsites",				/* TSRname	*/
    "proxy_efficiency",			/* PERname	*/
    {    				/* TTRname	*/
	"paf",
	"saf",
	"pdn",
	"sdn",
	"cct",
	"pst",
	"dld"
    },
    0,					/* Mode		*/
    0,					/* SYear	*/
    0L,					/* SDate	*/
    0,					/* EYear	*/
    SAMAXTIMET,				/* EDate	must be high enough */
    0,					/* LocDateLen	*/
    0,					/* dldCount	*/
    0,					/* utCount	*/
    0,					/* ueCount	*/
    0,					/* GF		*/
    NULL,				/* Downloads	*/
    NULL,				/* Usertab	*/
    "\000"				/* Period	*/
};

struct Localisation defloc = {
    "No",				/* No			*/
    "User",				/* User			*/
    "Requests",				/* Connections		*/
    "Seconds",				/* Seconds		*/
    "Bytes",				/* Bytes		*/
    "URL",				/* URL			*/
    "Period",				/* Period		*/
    "Sort field (order)",		/* SortFieldNOrder	*/
    "Users",				/* Users		*/
    "Served",				/* Served		*/
    "Actual",				/* Actual		*/
    "Cashed",				/* Cashed		*/
    "Traffic type",			/* TrafType		*/
    "Other local",			/* OthLocal		*/
    "Top user",				/* TopUser		*/
    "Top sites",			/* TopSites		*/
    "Proxy efficiency",			/* PEfficiency		*/
    "Date and Time",			/* DnT			*/
    "Date\\Time",			/* DsT			*/
    "Day of month",			/* DoM			*/
    "asc.",				/* Asc			*/
    "desc.",				/* Desc			*/
    "TOTAL",				/* TOTAL		*/
    "AVERAGE",				/* AVERAGE		*/
    "MAXIMUM",				/* MAXIMUM		*/
    "Code",				/* Code			*/
    "Generated by",			/* GeneratedBy		*/
    "Configuation",			/* Configuration	*/
    "Generation date",			/* GenerationD		*/
    "Period selection page",		/* PSP			*/
    {					/* TT[TTNUMBER]		*/
	"Proxy auth failures",
	"Site auth failures",
	"Proxy denied (ACL)",
	"Site denied",
	"CONNECT method",
	"PUT/POST method",
	"Downloads"
    },
    "PTGMk"				/* S			*/
};

struct Localisation *loc = &defloc;

struct GFConfig gfcfg = {
    0,					/* urlCount		*/
    0,					/* inameCount		*/
    0,					/* urlMode		*/
    0,					/* inameMode		*/
    NULL,				/* urls			*/
    NULL				/* inames		*/
};


#define F_CS2SI "CS2SI"
short int CS2SI(char *ivalue);
short int CS2SI(char *ivalue) {
    short int retval;
    
    if (sscanf(ivalue, "%hd", &retval) != 1) SAexit(F_CS2SI, "Error converting \"%s\" to short int", ivalue);
    
    return retval;
}


#define F_CS2C "CS2C"
char CS2C(char *ivalue);
char CS2C(char *ivalue) {
    if (ivalue[0] != 't' && ivalue[0] != 'f') SAexit(F_CS2C, "Error converting \"%s\" to char", ivalue);
    
    return (ivalue[0] == 't') ? 1 : 0;
}


#define F_LOADDOWNLOADS "LoadDownloads"
void LoadDownloads(void);
void LoadDownloads(void) {
    char *tBuf, *t;

    if (cfg.dldFile == NULL) SAexit(F_LOADDOWNLOADS, "Downloads report enabled, but no downloads file specified");
    if ((tBuf = (char *)SAfreadMC(NULL, cfg.dldFile)) == NULL) SAexit(F_LOADDOWNLOADS, "Can't stat file");
    
    while((t = strchr(tBuf, '\n')) != NULL) {
	icfg.Downloads = (char **)SArealloc(icfg.Downloads, sizeof(char *) * ((size_t)icfg.dldCount + 1L));
	icfg.Downloads[icfg.dldCount] = tBuf;
	t[0] = '\0';
	tBuf = t + 1L;

#if DEBUG > 0
	SAdebug(F_LOADDOWNLOADS, "%hd\t%s", icfg.dldCount + 1, icfg.Downloads[icfg.dldCount]);
#endif

	icfg.dldCount++;
    }

    /* TTR_DLD: disable if no extensions found */
    if (!icfg.dldCount) {
	SAwarning(F_LOADDOWNLOADS, "Empty file");
	cfg.TTR[DLD] = 0;
    }
}


#define F_LOADUSERTAB "LoadUsertab"
void LoadUsertab(void);
void LoadUsertab(void) {
    char *tBuf, *t;

    if ((tBuf = (char *)SAfreadMC(NULL, cfg.utFile)) == NULL) SAexit(F_LOADUSERTAB, "Can't stat file");
    
    while((t = strchr(tBuf, ' ')) != NULL) {
	icfg.Usertab = (struct SAuconv *)SArealloc(icfg.Usertab, sizeof(struct SAuconv) * ((size_t)icfg.utCount + 1L));
	icfg.Usertab[icfg.utCount].iname = tBuf;
	t[0] = '\0';
	tBuf = t + 1L;
	
	if ((t = strchr(tBuf, '\n')) == NULL) SAexit(F_LOADUSERTAB, "Incorrect file format");
	t[0] = '\0';
	icfg.Usertab[icfg.utCount].ename = tBuf;
	tBuf = t + 1L;
	
#if DEBUG > 0
	SAdebug(F_LOADUSERTAB, "%hd\t%s->%s", icfg.utCount + 1, icfg.Usertab[icfg.utCount].iname, icfg.Usertab[icfg.utCount].ename);
#endif

	icfg.utCount++;
    }

    if (icfg.utCount) qsort(icfg.Usertab, (size_t)icfg.utCount, sizeof(struct SAuconv), (int (*)(const void *, const void *))compSAuconv);
    else SAwarning(F_LOADUSERTAB, "Empty file");
}


#define F_LOADLOCALISATION "LoadLocalisation"
void LoadLocalisation(void);
void LoadLocalisation(void) {
    char *tBuf, *t;
    static char *newloc[LOCSIZE];
    short int locCount = 0;

    if ((tBuf = (char *)SAfreadMC(NULL, cfg.locFile)) == NULL) SAexit(F_LOADLOCALISATION, "Can't stat file");
    
    while((t = strchr(tBuf, '\n')) != NULL) {
	newloc[locCount] = tBuf;
	t[0] = '\0';
	tBuf = t + 1L;

#if DEBUG > 0
	SAdebug(F_LOADLOCALISATION, "%hd\t%s", locCount + 1, newloc[locCount]);
#endif

	locCount++;
    }

    if (locCount != LOCSIZE) SAexit(F_LOADLOCALISATION, "Incorrect file format");
    
    loc = (struct Localisation *)((char **)(&newloc));
}


#define F_LOADGLOBALFILTER "LoadGlobalFilter"
void LoadGlobalFilter(void);
void LoadGlobalFilter(void) {
    char *tBuf, *t;

    if ((tBuf = (char *)SAfreadMC(NULL, cfg.gfFile)) == NULL) SAexit(F_LOADGLOBALFILTER, "Can't stat file");
    
    while((t = strchr(tBuf, '\n')) != NULL) {
	t[0] = '\0';
	switch(tBuf[0]) {
	    case 'i':
		    gfcfg.inames = (char **)SArealloc(gfcfg.inames, sizeof(char *) * (gfcfg.inameCount + 1L));
		    gfcfg.inames[gfcfg.inameCount] = &tBuf[2];
		    gfcfg.inameCount++;
		    break;
	    case 'I':
		    if (tBuf[2] >= '0' && tBuf[2] <= '3') gfcfg.inameMode = tBuf[2] - '0';
		    else SAexit(F_LOADGLOBALFILTER, "Incorrect file format");
		    break;
	    case 'u':
		    gfcfg.urls = (char **)SArealloc(gfcfg.urls, sizeof(char *) * (gfcfg.urlCount + 1L));
		    gfcfg.urls[gfcfg.urlCount] = &tBuf[2];
		    gfcfg.urlCount++;
		    break;
	    case 'U':
		    if (tBuf[2] >= '0' && tBuf[2] <= '3') gfcfg.urlMode = tBuf[2] - '0';
		    else SAexit(F_LOADGLOBALFILTER, "Incorrect file format");
		    break;
	    case '#':
	    case '\n':
		    break;
	    default:
		    SAexit(F_LOADGLOBALFILTER, "Incorrect file format");
		    break;
	}
#if DEBUG > 0
	SAdebug(F_LOADGLOBALFILTER, "%c\t%s", tBuf[0], &tBuf[2]);
#endif
	tBuf = t + 1L;
    }
    
    
    if (gfcfg.inameCount || gfcfg.urlCount) {
	if (gfcfg.inameMode > 1) qsort(gfcfg.inames, (size_t)gfcfg.inameCount, sizeof(char *), (int (*)(const void *, const void *))SAstrcmp);
	if (gfcfg.urlMode > 1) qsort(gfcfg.urls, (size_t)gfcfg.urlCount, sizeof(char *), (int (*)(const void *, const void *))SAstrcmp);
	icfg.GF = 1;
    } else SAwarning(F_LOADGLOBALFILTER, "Empty file");
}


#define F_SAUSAGE "SAusage"
void SAusage(char *iName, char mode, char *itext);
void SAusage(char *iName, char mode, char *itext) {
    const char *const utext = "\
Usage: %s [-s|-r|-F] [-d STARTDATE-ENDDATE] [-f FILE] [-h] [-l FILE] [-v]\n\
  -d STARTDATE-ENDDATE	Dates range for report generation in current locale,\n\
			you may specify only STARTDATE followed by '-' sign,\n\
			or only ENDDATE preceded by '-' sign.\n\
  -f FILE		Read configuration from FILE.\n\
  -h			Show help message\n\
  -l FILE		Use FILE as log file.\n\
  -s			Show log file information:\n\
			name, size, start and end dates.\n\
  -r			Remove records from log file.\n\
			You must also specify date(s) at '-d' option.\n\
  -v			Show version\n\
  -F			Fix root index.html file according to\n\
			reports directories presense.\n";
    char etext[MAXLEN];
    size_t elen = 0;

    if (mode == 0) {
	SAshow(utext, iName);
	exit(EXIT_SUCCESS);
    } else {
	switch(mode) {
	    case 1:
		    elen = snprintf(etext, sizeof(etext), "Incorrect option '%s'\n", itext);
		    break;
	    case 2:
		    elen = snprintf(etext, sizeof(etext), "Incorrect dates range specified '%s'\n", itext);
		    break;
	    case 3:
		    elen = snprintf(etext, sizeof(etext), "Option '%s' requires argument\n", itext);
		    break;
	    case 4:
		    elen = snprintf(etext, sizeof(etext), "Too much action options specified\n");
		    break;
	}
	
	snprintf(&etext[elen], MAXLEN - elen, utext, iName);
	SAexit(F_SAUSAGE, etext);
    }
}


#define F_INITCONFIGWITHCL "InitConfigWithCL"
void InitConfigWithCL(int argc, char **argv) {
    short int i = 1;

    
    /* Parse command line options */
    if (argc > 1)
	for (; i < argc; i++) {
	    if (argv[i][0] != '-' || argv[i][2] != '\0') SAusage(argv[0], 1, argv[i]);
	    switch(argv[i][1]) {
		case 'f':
			if (i == argc - 1) SAusage(argv[0], 3, argv[i]);
			cfg.Config = argv[i + 1];
			i++; /* 2 args readed for this option */
			break;
		case 'd':
			if (i == argc - 1) SAusage(argv[0], 3, argv[i]);
			if (argv[i + 1][0] == '-') icfg.EDate = GetDZT(&argv[i + 1][1]) + 24L * 60L * 60L - 1L;
			else {
			    if (argv[i + 1][icfg.LocDateLen] != '-') SAusage(argv[0], 2, argv[i + 1]);
			    argv[i + 1][icfg.LocDateLen] = '\0';
			    icfg.SDate = GetDZT(&argv[i + 1][0]);
			    if (argv[i + 1][icfg.LocDateLen + 1] != '\0') icfg.EDate = GetDZT(&argv[i + 1][icfg.LocDateLen + 1]) + 24L * 60L * 60L - 1L;
			}
			i++; /* 2 args readed for this option */
			break;
		case 'h':
			SAusage(argv[0], 0, NULL);
			break;
		case 'l':
			if (i == argc - 1) SAusage(argv[0], 3, argv[i]);
			owcfg.Log = argv[i + 1];
			i++; /* 2 args readed for this option */
			break;
		case 's':
			if (icfg.Mode != 0) SAusage(argv[0], 4, NULL);
			else icfg.Mode = 2;
			break;
		case 'r':
			if (icfg.Mode != 0) SAusage(argv[0], 4, NULL);
			else icfg.Mode = 3;
			break;
		case 'v':
			SAshow("%s %s (%s)\n", PROGNAME, PROGVERSION, PROGURL);
			exit(0);
			break;
		case 'F':
			if (icfg.Mode != 0) SAusage(argv[0], 4, NULL);
			else icfg.Mode = 4;
			break;
		default:
			SAusage(argv[0], 1, argv[i]);
			break;
	    }
	}

    InitConfig();
}


#define F_INITCONFIG "InitConfig"
void InitConfig(void) {
    FILE *CONF;
    char tBuf[128];
    char *value;
    

    /* Open configuration file */
    CONF = SAfopenI("r", cfg.Config);
    
    /* Read and parse configuration file */
    while (fgets(tBuf, 128, CONF) != NULL) {
	/* Skip comments and empty lines */
	if (tBuf[0] == '\n' || tBuf[0] == '#') continue;
	
	/* Divide for parameter and value at =, remove " from value */
	if ((value = strchr(tBuf, '=')) == NULL) SAexit(F_INITCONFIG, "Error parsing confuguration file, line was: %s", tBuf);
	value[0] = '\0';
	value += 2L;
	value[strlen(value) - 2L] = '\0';

#if DEBUG>0	
	SAdebug(F_INITCONFIG, "%s=\"%s\"", tBuf, value);
#endif
	
	switch(tBuf[0]) {
	    case 'a':
		    if (strncmp(tBuf, "autorecords", 11L) == 0) cfg.AutoRecords = CS2SI(value);
		    break;
	    case 'c':
		    if (strncmp(tBuf, "cct_limit", 9L) == 0) cfg.ttUserLimit[CCT] = CS2SI(value);
		    else if (strncmp(tBuf, "cct", 3L) == 0) cfg.TTR[CCT] = CS2C(value);
		    break;
	    case 'd':
		    if (strncmp(tBuf, "dld_limit", 9L) == 0) cfg.ttUserLimit[DLD] = CS2SI(value);
		    else if (strncmp(tBuf, "dld", 3L) == 0) cfg.TTR[DLD] = CS2C(value);
		    else if (strncmp(tBuf, "downloads", 9L) == 0) cfg.dldFile = SAstrcpy(value);
		    break;
	    case 'f':
		    if (strncmp(tBuf, "fulltraffic", 11L) == 0) cfg.FullTraffic = CS2C(value);
		    else if (strncmp(tBuf, "fullurl", 7L) == 0) cfg.FullURL = CS2C(value);
		    break;
	    case 'g':
		    if (strncmp(tBuf, "global_filter", 13L) == 0) cfg.gfFile = SAstrcpy(value);
		    break;
	    case 'i':
		    if (strncmp(tBuf, "inameuser", 9L) == 0) cfg.INameUser = CS2C(value);
		    else if (strncmp(tBuf, "indicators", 10L) == 0) cfg.Indics = CS2C(value);
		    break;
	    case 'l':
		    if (strncmp(tBuf, "locale", 6L) == 0) cfg.Locale = SAstrcpy(value);
		    else if (strncmp(tBuf, "logformat", 9L) == 0) cfg.LogFormat = CS2SI(value);
		    else if (strncmp(tBuf, "logo", 4L) == 0) cfg.Logo = SAstrcpy(value);
		    else if (strncmp(tBuf, "log", 3L) == 0) cfg.Log = SAstrcpy(value);
		    else if (strncmp(tBuf, "localisation", 12L) == 0) cfg.locFile = SAstrcpy(value);
		    break;
	    case 'n':
		    if (strncmp(tBuf, "name", 4L) == 0) cfg.Name = SAstrcpy(value);
		    break;
	    case 'o':
		    if (strncmp(tBuf, "overwrite", 9L) == 0) cfg.Overwrite = CS2C(value);
		    break;
	    case 'p':
		    if (strncmp(tBuf, "paf_limit", 9L) == 0) cfg.ttUserLimit[PAF] = CS2SI(value);
		    else if (strncmp(tBuf, "paf", 3L) == 0) cfg.TTR[PAF] = CS2C(value);
		    else if (strncmp(tBuf, "pdn_limit", 9L) == 0) cfg.ttUserLimit[PDN] = CS2SI(value);
		    else if (strncmp(tBuf, "pdn", 3L) == 0) cfg.TTR[PDN] = CS2C(value);
		    else if (strncmp(tBuf, "pst_limit", 9L) == 0) cfg.ttUserLimit[PST] = CS2SI(value);
		    else if (strncmp(tBuf, "pst", 3L) == 0) cfg.TTR[PST] = CS2C(value);		    
		    break;
	    case 'r':
		    if (strncmp(tBuf, "resolveip", 9L) == 0) cfg.ResolveIP = CS2C(value);
		    break;
	    case 's':
		    if (strncmp(tBuf, "saf_limit", 9L) == 0) cfg.ttUserLimit[SAF] = CS2SI(value);
		    else if (strncmp(tBuf, "saf", 3L) == 0) cfg.TTR[SAF] = CS2C(value);
		    else if (strncmp(tBuf, "sdn_limit", 9L) == 0) cfg.ttUserLimit[SDN] = CS2SI(value);
		    else if (strncmp(tBuf, "sdn", 3L) == 0) cfg.TTR[SDN] = CS2C(value);
		    else if (strncmp(tBuf, "showinfo", 8L) == 0) cfg.ShowInfo = CS2C(value);
		    else if (strncmp(tBuf, "site", 4L) == 0) cfg.Site = SAstrcpy(value);
		    break;
	    case 't':
		    if (strncmp(tBuf, "targetdir", 9L) == 0) cfg.TargetDir = SAstrcpy(value);
		    else if (strncmp(tBuf, "tmpdir", 7L) == 0) cfg.TmpDir = SAstrcpy(value);
		    else if (strncmp(tBuf, "ts", 2L) == 0) cfg.TSR = CS2C(value);
		    break;
	    case 'u':
		    if (strncmp(tBuf, "url_limit", 9L) == 0) cfg.URLLimit = CS2SI(value);
		    else if (strncmp(tBuf, "usertab", 7L) == 0) cfg.utFile = SAstrcpy(value);
		    break;
	    default:
		    /* Not for all cases, try variable beginning with some listed above char */
		    SAexit(F_INITCONFIG, "Unknown parameter \"%s\" with value \"%s\"", tBuf, value);
		    break;
	}
    }

    /* Close configuration file */
    SAfclose(CONF, cfg.Config);

    /* Command line options arguments is preferred over config file */
    if (owcfg.Log != NULL) cfg.Log = owcfg.Log;

    /* If alternative mode not selected at CL, then choose log processing */
    if (icfg.Mode == 0) icfg.Mode = 1;

    /* If alternative locale specified, reinitialise it */
    if (cfg.Locale[0] != '\0') InitLocale();

    /* If no configuration name specified, then use config file name */
    if (cfg.Name == NULL) cfg.Name = cfg.Config;

    /* Load files, which configuration depend */
    if (cfg.TTR[DLD]) LoadDownloads();
    if (cfg.utFile != NULL) LoadUsertab();
    if (cfg.locFile != NULL) LoadLocalisation();
    if (cfg.gfFile != NULL) LoadGlobalFilter();
}


#define F_INITLOCALE "InitLocale"
void InitLocale(void) {
    time_t lcdate = 1135976400;		/* 31.12.2005 - we assume it's longest date in any locale */
    char locstring[MAXFILENAME];
    
    /* Init locale */
    if (setlocale(LC_ALL, cfg.Locale) == NULL) SAexit(F_INITLOCALE, "Can't initialize locale");
    icfg.CharSet = nl_langinfo(CODESET);
    
    snprintf(locstring, sizeof(locstring), "LC_ALL=%s", cfg.Locale);
    if (putenv(locstring)) SAexit(F_INITLOCALE, "Can't set LC_ALL to %s", locstring);
    if (putenv("LC_COLLATE=POSIX")) SAexit(F_INITLOCALE, "Can't set LC_COLLATE to POSIX");

    /* Compute max date length */    
    icfg.LocDateLen = strlen(GetDT(lcdate, "%x"));
}
