#include "global.h"


#if DEBUG != 0
void SAdebug(const char *func, const char *fmt, ...) {
    va_list args;
    
    va_start(args, fmt);
    fprintf(stderr, "SADEBUG: [%s] ", func);
    vfprintf(stderr, fmt, args);
    fprintf(stderr, "\n");
    fflush(stderr);
    va_end(args);
}
#endif


void SAshow(const char *fmt, ...) {
    va_list args;
    
    va_start(args, fmt);
    vfprintf(stdout, fmt, args);
    va_end(args);
}


void SAwarning(const char *func, const char *fmt, ...) {
    va_list args;
    
    va_start(args, fmt);
    fprintf(stderr, "SAWARNING: [%s] ", func);
    vfprintf(stderr, fmt, args);
    fprintf(stderr, "\n");
    fflush(stderr);
    va_end(args);
}


void SAexit(const char *func, const char *fmt, ...) {
    va_list args;
    
    va_start(args, fmt);
    fprintf(stderr, "SAERROR: [%s] ", func);
    vfprintf(stderr, fmt, args);
    fprintf(stderr, "\n");
    va_end(args);
		    
    exit(EXIT_FAILURE);
}


#define F_SASYSTEM "SAsystem"
void SAsystem(const char *program, ...) {
    pid_t tpid;
    int tmp = 0, status;
    va_list args;
    char *argsarray[MAXARGS];

    tpid = fork();
    if (tpid == 0) {
	va_start(args, program);
	
	for (argsarray[0] = SAstrcpy(program); (argsarray[tmp] = va_arg(args, char *)) != NULL; tmp++);
    
#if DEBUG >= 2
	SAdebug (F_SASYSTEM, program);
#endif
	execv(program, argsarray);
	SAexit(F_SASYSTEM, "Can't execute %s", program);
    }

    if (waitpid(tpid, &status, 0) < 0) SAexit(F_SASYSTEM, "waitpid %s, %s", program, strerror(errno));;
    if (WIFSIGNALED(status)) SAexit(F_SASYSTEM, "%s got signal", program);
    if (WIFEXITED(status)) if (!WEXITSTATUS(status)) return;
    
    /* Some error during execution occured */
    SAexit(F_SASYSTEM, "%s returned %d", program, WEXITSTATUS(status));
}


#define F_SAMALLOC "SAmalloc"
void *SAmalloc(size_t size) {
    void *retval;
    
    retval = malloc(size);
    if (retval == NULL) SAexit(F_SAMALLOC, "No free memory available");
    
    return retval;
}


#define F_SAREALLOC "SArealloc"
void *SArealloc(void *ptr, size_t size) {
    void *retval;
    
    retval = realloc(ptr, size);
    if (retval == NULL) SAexit(F_SAREALLOC, "No free memory available");
    
    return retval;
}


char *SAstrcpy(const char *src) {
    return strcpy((char *)SAmalloc(strlen(src) + 1L), src);
}


int SAstrcmp(const char **el1, const char **el2) {
    return strcmp(*el1, *el2);
}
